@@ -1,28 +1,30 @@
 
 import inspect
 import ast
-import warnings
-import arviz as az
-import seaborn as sns
-import matplotlib.pyplot as plt
-import numpyro
-import time as tm
-from jax import jit
-from jax import vmap
-import jax.numpy as jnp
-import jax as jax
-import numpy as np
+
 import jax.random as random
-import numpy as np
+import jax.numpy as jnp
+from jax import vmap
+from jax import jit
+import jax 
+
+from numpyro.infer import MCMC, NUTS, Predictive
+from numpyro.handlers import condition
+import numpyro
+
+import arviz as az
+
+
 import random as pyrand
+import functools
 
 from BI.Data.manip import manip
 from BI.Utils.array import Mgaussian as gaussian
-from BI.Utils.array import factors 
+from BI.Utils.array import effects 
 from BI.SetDevice.set import setup_device
 
 from BI.Utils.link import link
-from BI.Diagnostic.Diag import diag
+from BI.Diagnostic.Diag2 import diagWIP as diag
 from BI.Network.Net import net
 from BI.NBDA.NBDA import NBDA
 from BI.Models.surv import survival
@@ -31,12 +33,11 @@ from BI.Models.DPMM import *
 from BI.Models.models import models
 from BI.ML.ml import ml
 from BI.BNN.bnn import bnn 
-from numpyro.infer import MCMC, NUTS, Predictive
-from numpyro.handlers import condition
+
 
 
 class bi(manip):
-    def __init__(self, platform='cpu', cores=None, deallocate = False, print_devices_found =    True, backend='numpyro'):
+    def __init__(self, platform='cpu', cores=None, rand_seed = True, deallocate = False, print_devices_found = True, backend='numpyro'):
         """
         Initialize the BI class with platform, cores, deallocate, print_devices_found, and backend parameters.
         
@@ -50,6 +51,11 @@ class bi(manip):
         manip.__init__(self)
         setup_device(platform, cores, deallocate, print_devices_found) 
         
+        if rand_seed:
+            self.seed = rand_seed           
+        else: 
+            self.seed = rand_seed
+
         self.data_on_model = None
         self.priors_name = None
         self.tab_summary = None
@@ -63,7 +69,7 @@ class bi(manip):
 
         self.gaussian = gaussian
         self.survival = survival(self)
-        self.factor = factors
+        self.effects = effects
         self.link = link
 
         self.dpmm = dpmm
@@ -78,8 +84,8 @@ class bi(manip):
         self.bnn= bnn()  
 
         if backend == 'numpyro':
-            from BI.Utils.np_dists import UnifiedDist as np_dists
-            self.dist=np_dists 
+            from BI.Utils.np_dists2 import UnifiedDist as np_dists
+            self.dist=np_dists(self.seed)
             jax.config.update("jax_enable_x64", True)
 
         elif backend == 'tfp':
@@ -89,6 +95,7 @@ class bi(manip):
             self.sampler = mcmc_tfp()
             jax.config.update("jax_enable_x64", False)
 
+
     def fit(self, 
             model = None, 
             obs=None,
@@ -146,6 +153,12 @@ class bi(manip):
             jit_model_args: Whether to JIT model arguments.
             seed: Random seed.
         """
+        if obs is None:
+            obs = self.data_on_model
+        else:
+            self.data_on_model = obs
+
+
         if model is None:
             if self.nbdaModel == False:
                 print( "Argument model can't be None")
@@ -155,7 +168,10 @@ class bi(manip):
                 self.model_name = 'NBDA' 
         else:
             self.model = model
-            self.model_name = model.__name__
+            if isinstance(model, functools.partial):
+                self.model_name = model.func.__name__
+            else:
+                self.model_name = model.__name__
 
         if self.nbdaModel == False:
             if self.data_on_model is None :
@@ -169,6 +185,26 @@ class bi(manip):
                 self.ml.KMEANS(self.data_on_model['data'], n_clusters=self.data_on_model['K'])
                 self.data_on_model['initial_means'] = self.ml.results['centroids']
 
+        if self.model_name == 'pca':
+            self.model = model.get_model(model.type)
+            tmp = self.data_on_model.keys()
+
+            if 'X' not in tmp:
+                return print('X is required')
+            else:
+                self.data_on_model['X']=self.data_on_model['X'].T
+            if 'data_dim' not in tmp:
+                self.data_on_model['data_dim'] = self.data_on_model['X'].shape[0]
+            if 'latent_dim' not in tmp:
+                self.data_on_model['latent_dim'] = self.data_on_model['X'].shape[0]
+            if 'num_data_points' not in tmp:
+                self.data_on_model['num_data_points'] = self.data_on_model['X'].shape[1]
+            # Init pca class with data
+            self.models.pca=self.models.pca(
+                X = self.data_on_model['X'], 
+                latent_dim = self.data_on_model['latent_dim'],
+                type = model.type)  
+
         if self.backend == 'numpyro':
             from BI.Samplers.mcmc_numpyro import mcmc_numpyro
             self.sampler = mcmc_numpyro(
@@ -220,7 +256,11 @@ class bi(manip):
                 trace[name] = samp
             self.posteriors = trace
             self.get_history()
+        if self.model_name == 'pca':
+            self.models.pca.posterior = self.posteriors
+            self.models.pca.get_attributes(self.models.pca.X.T)
 
+    # Random number generator ----------------------------------------------------------------
     def randint(self, low, high, shape):
         """
         Generate random integers in the given range.
@@ -236,7 +276,7 @@ class bi(manip):
         return pyrand.randint(low, high, shape)
 
     # Get posteriors ----------------------------------------------------------------------------
-    def summary(self, round_to=2, kind="stats", hdi_prob=0.89, *args, **kwargs): 
+    def summary(self, round_to=2, kind="all", hdi_prob=0.89, *args, **kwargs): 
         """
         Generate a summary of the posterior distribution.
         
@@ -436,3 +476,5 @@ class bi(manip):
             plot_gmm(X, sampler= self.sampler,figsize=figsize,**kwargs)
         if self.model_name == 'dpmm':
             plot_dpmm(X, sampler= self.sampler,figsize=figsize,**kwargs)
+        else:
+            print('Model not supported')
