@@ -7,6 +7,7 @@ import os
 import sys
 import inspect
 from BI.Utils.np_dists import UnifiedDist as dist
+from BI.Utils.link import link 
 
 class Neteffect(array_manip):
     """Neteffect class for managing and computing network effects in Bayesian models.
@@ -37,9 +38,9 @@ class Neteffect(array_manip):
     # Sender receiver  ----------------------
     @staticmethod 
     def nodes_random_effects(N_id, sr_mu = 0, sr_sd = 1, sr_sigma_rate = 1, cholesky_dim = 2, cholesky_density = 2, sample = False, diag = False ):
-        sr_raw =  dist.normal(sr_mu, sr_sd, shape=(2, N_id), name = 'sr_raw', sample = sample)
-        sr_sigma =  dist.exponential( sr_sigma_rate, shape= (2,), name = 'sr_sigma', sample = sample)
-        sr_L = dist.lkj_cholesky(cholesky_dim, cholesky_density, name = "sr_L", sample = sample)
+        sr_raw =  dist.normal(sr_mu, sr_sd, shape=(2, N_id), name = 'sr_raw', sample = sample, to_jax=True)
+        sr_sigma =  dist.exponential( sr_sigma_rate, shape= (2,), name = 'sr_sigma', sample = sample, to_jax=True)
+        sr_L = dist.lkj_cholesky(cholesky_dim, cholesky_density, name = "sr_L", sample = sample, to_jax=True)
         rf = deterministic('sr_rf',(((sr_L @ sr_raw).T * sr_sigma)))
 
         if diag:
@@ -52,9 +53,12 @@ class Neteffect(array_manip):
             print("rf--------------------------------------------------------------------------------")
             print(rf)
         return rf, sr_raw, sr_sigma, sr_L
+    
     @staticmethod 
     def nodes_terms(focal_individual_predictors, target_individual_predictors,
-                    N_var = 1, s_mu = 0, s_sd = 1, r_mu = 0, r_sd = 1, sample = False, diag = False, focal_name='focal_effects', target_name ='target_effects' ):
+                    N_var_focal = 1,
+                    N_var_target = 1,
+                    s_mu = 0, s_sd = 1, r_mu = 0, r_sd = 1, sample = False, diag = False, focal_name='focal_effects', target_name ='target_effects' ):
         """_summary_
 
         Args:
@@ -69,9 +73,13 @@ class Neteffect(array_manip):
         Returns:
             _type_: terms, focal_effects, target_effects
         """
-        focal_effects = dist.normal(s_mu, s_sd, shape=(N_var,), sample = sample, name = focal_name)
-        target_effects =  dist.normal( r_mu, r_sd, shape= (N_var,), sample = sample, name = target_name)
-        terms = jnp.stack([focal_effects @ focal_individual_predictors, target_effects @  target_individual_predictors], axis = -1)
+        A_f = jnp.ones((N_var_focal,))
+        A_f = A_f.at[0].set(0)
+        A_t = jnp.ones((N_var_focal,))
+        A_t = A_t.at[0].set(0)
+        focal_effects = dist.normal(s_mu, s_sd, shape=(N_var_focal,), sample = sample, name = focal_name, to_jax=True)
+        target_effects =  dist.normal( r_mu, r_sd, shape= (N_var_target,), sample = sample, name = target_name, to_jax=True)
+        terms = jnp.stack([(focal_effects*A_f) @ focal_individual_predictors, (target_effects*A_t) @  target_individual_predictors], axis = -1)
 
         if diag:
             print("focal_effects--------------------------------------------------------------------------------")
@@ -98,12 +106,16 @@ class Neteffect(array_manip):
         """
         ids = jnp.arange(0,sr_effects.shape[0])
         edgl_idx = Neteffect.vec_node_to_edgle(jnp.stack([ids, ids], axis = -1))
-        sender = sr_effects[edgl_idx[:,0],0] + sr_effects[edgl_idx[:,1],1]
-        receiver = sr_effects[edgl_idx[:,1],0] + sr_effects[edgl_idx[:,0],1]
-        return jnp.stack([sender, receiver], axis = 1)
+        S_i = sr_effects[edgl_idx[:,0],0]
+        S_j = sr_effects[edgl_idx[:,1],0]
+        R_i = sr_effects[edgl_idx[:,0],1]
+        R_j = sr_effects[edgl_idx[:,1],1]
+        return jnp.stack([S_i + R_j, S_j + R_i ], axis = 1)
 
     @staticmethod 
-    def sender_receiver(focal_individual_predictors, target_individual_predictors,  s_mu = 0, s_sd = 1, r_mu = 0, r_sd = 1, #Fixed effect parameters
+    def sender_receiver(focal_individual_predictors, target_individual_predictors,  
+                        s_mu = 0, s_sd = 1, 
+                        r_mu = 0, r_sd = 1, #Fixed effect parameters
                         sr_mu = 0, sr_sd = 1, sr_sigma_rate = 1, cholesky_dim = 2, cholesky_density = 2, #Random effect parameters
                         sample = False, diag = False ):
         """Compute sender-receiver effects combining both fixed and random effects.
@@ -126,12 +138,19 @@ class Neteffect(array_manip):
         Returns:
             jax array: Combined dyadic effects.
         """                            
-        N_var = focal_individual_predictors.shape[0]
+        N_var_focal = focal_individual_predictors.shape[0]
+        N_var_target= target_individual_predictors.shape[0]
         N_id = focal_individual_predictors.shape[1]            
 
-        sr_ff, focal_effects, target_effects = Neteffect.nodes_terms(focal_individual_predictors, target_individual_predictors, N_var = N_var, s_mu = s_mu, s_sd = s_sd, r_mu = r_mu, r_sd = r_sd, sample = sample, diag = diag )
+        sr_ff, focal_effects, target_effects = Neteffect.nodes_terms(
+            focal_individual_predictors, 
+            target_individual_predictors, 
+            N_var_focal = N_var_focal, 
+            N_var_target = N_var_target, 
+            s_mu = s_mu, s_sd = s_sd, r_mu = r_mu, r_sd = r_sd, sample = sample, diag = diag )
 
         sr_rf, sr_raw, sr_sigma, sr_L = Neteffect.nodes_random_effects(N_id, sr_mu = sr_mu, sr_sd = sr_sd, sr_sigma_rate = sr_sigma_rate, cholesky_dim = cholesky_dim, cholesky_density = cholesky_density,  sample = sample, diag = diag ) # shape = N_id
+
         sr_to_dyads = Neteffect.node_effects_to_dyadic_format(sr_ff + sr_rf) # sr_ff and sr_rf are nodal values that need to be converted to dyadic values
         return sr_to_dyads
 
@@ -169,9 +188,9 @@ class Neteffect(array_manip):
         Returns:
             tuple: Contains random effects, raw effects, sigma, and Cholesky decomposition matrix.
         """
-        dr_raw =  dist.normal(dr_mu, dr_sd, shape=(2,N_dyads), name = 'dr_raw', sample = sample)
-        dr_sigma = dist.exponential(dr_sigma, shape=(1,), name = 'dr_sigma', sample = sample )
-        dr_L = dist.lkj_cholesky(cholesky_dim, cholesky_density, name = 'dr_L', sample = sample)
+        dr_raw =  dist.normal(dr_mu, dr_sd, shape=(2,N_dyads), name = 'dr_raw', sample = sample, to_jax=True)
+        dr_sigma = dist.exponential(dr_sigma, shape=(1,), name = 'dr_sigma', sample = sample, to_jax=True )
+        dr_L = dist.lkj_cholesky(cholesky_dim, cholesky_density, name = 'dr_L', sample = sample, to_jax=True)
         dr_rf = deterministic('dr_rf', (((dr_L @ dr_raw).T * jnp.repeat(dr_sigma, 2))))
         if diag :
             print("dr_raw--------------------------------------------------------------------------------")
@@ -198,24 +217,14 @@ class Neteffect(array_manip):
         Returns:
             tuple: Contains fixed effects and dyadic predictors.
         """        
-        dyad_effects = dist.normal(d_m, d_sd, name= 'dyad_effects', shape = (dyadic_predictors.ndim - 1,), sample = sample)
-        
-        if dyadic_predictors.ndim == 2:
-            dr_ff = dyad_effects * dyadic_predictors
-            if diag :
-                print("dyad_effects--------------------------------------------------------------------------------")
-                print(dyad_effects)
-                print("rf--------------------------------------------------------------------------------")
-                print(rf)
-            return dr_ff, dyad_effects
-        else:
-            if diag :
-                print("dyad_effects--------------------------------------------------------------------------------")
-                print(dyad_effects)
-                print("rf--------------------------------------------------------------------------------")
-                print(rf)
-            dr_ff = dyadic_predictors * dyad_effects[:,None, None]
-            return jnp.sum(dr_ff, axis=0), dyad_effects
+        if dyadic_predictors.ndim != 3:
+            print('Error: Argument dyadic_predictors must be a 3D array')
+        third_axis = dyadic_predictors.shape[2]
+        A_d = jnp.ones((third_axis,))
+        A_d = A_d.at[0].set(0)
+        dyad_effects = dist.normal(d_m, d_sd, name= 'dyad_effects', shape = (third_axis,), sample = sample, to_jax=True)
+        dr_ff = (dyad_effects * A_d) * dyadic_predictors
+        return jnp.sum(dr_ff, axis=2), dyad_effects
 
     @staticmethod 
     def dyadic_effect(dyadic_predictors = None, shape = None, d_m = 0, d_sd = 1, # Fixed effect arguments
@@ -251,7 +260,6 @@ class Neteffect(array_manip):
             cholesky_dim = cholesky_dim, cholesky_density = cholesky_density, sample = sample)
         return  dr_rf
   
-  
     @staticmethod 
     def block_model_prior(N_grp, 
                           b_ij_mean = 0.01, b_ij_sd = 2.5, 
@@ -270,7 +278,7 @@ class Neteffect(array_manip):
             _type_: _description_
         """
         N_dyads = int(((N_grp*(N_grp-1))/2))
-        b_ij = dist.normal(Neteffect.logit(b_ij_mean/jnp.sqrt(N_grp*0.5 + N_grp*0.5)), b_ij_sd, shape=(N_dyads, 2), name = name_b_ij, sample = sample) # transfers more likely within groups
+        b_ij = dist.normal(Neteffect.logit(b_ij_mean/jnp.sqrt(N_grp*0.5 + N_grp*0.5)), b_ij_sd, shape=(N_dyads, 2), name = name_b_ij, sample = sample, to_jax=True) # transfers more likely within groups
         b_ii = dist.normal(Neteffect.logit(b_ii_mean/jnp.sqrt(N_grp)), b_ii_sd, shape=(N_grp, ), name = name_b_ii, sample = sample) # transfers less likely between groups
         b = Neteffect.edgl_to_mat(b_ij, N_grp)
         b = b.at[jnp.diag_indices_from(b)].set(b_ii)
@@ -278,7 +286,7 @@ class Neteffect(array_manip):
 
     @staticmethod 
     @jit
-    def block_prior_to_edglelist(v, b):
+    def block_prior_to_edglelist(b, v ):
         """Convert block vector id group belonging to edgelist of i->j group values
 
         Args:
@@ -289,7 +297,6 @@ class Neteffect(array_manip):
             _type_: 1D array representing the probability of links from i-> j 
         """
 
-        v = Neteffect.vec_node_to_edgle(jnp.stack([v, v], axis= 1)).astype(int)
         return jnp.stack([b[v[:,0],v[:,1]], b[v[:,1],v[:,0]]], axis = 1)
 
     @staticmethod 
@@ -318,6 +325,7 @@ class Neteffect(array_manip):
         name_b_ij = 'b_ij_' + str(name)
         name_b_ii = 'b_ii_' + str(name) 
 
+
         b, b_ij, b_ii = Neteffect.block_model_prior(N_grp, 
                          b_ij_mean = b_ij_mean, b_ij_sd = b_ij_sd, 
                          b_ii_mean = b_ii_mean, b_ii_sd = b_ii_sd,
@@ -326,3 +334,11 @@ class Neteffect(array_manip):
 
         return edgl_block
 
+
+    @staticmethod
+    def block_model2(group, N_group, N_by_group,  b_ij_sd = 2.5, sample = False, name = ''): 
+        base_rate = jnp.tile(0.01, (N_group,N_group))
+        base_rate = base_rate.at[jnp.diag_indices_from(base_rate)].set(0.1)
+        mu_ij = base_rate/jnp.sqrt(jnp.outer(N_by_group, N_by_group))
+        b = dist.normal(Neteffect.logit(mu_ij), b_ij_sd, sample = sample, name = f'b_{name}')
+        return Neteffect.block_prior_to_edglelist(b, group)
\ No newline at end of file
